#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License. See accompanying LICENSE file.
#

#!/usr/bin/env python

import argparse
import json

# Create the parser
my_parser = argparse.ArgumentParser(description='Create test Ranger Service policy')

# Add the arguments
my_parser.add_argument('--inputPath',   metavar='policy-input-directory', type=str, help='Policy Input directory path', required=True)
my_parser.add_argument('--outputPath',  metavar='policy-output-directory', type=str, help='Policy Output directory path', required=True)
my_parser.add_argument('--policyItems', metavar='Policy item count', type=int, help='Number of dummy policy items to be stuffed', required=True)

policy_item_hive_template = '{"service":"SERVICE_NAME","name":"POLICY_NAME","isAuditEnabled":false,"resources":{"database":{"values":["*"],"isExcludes":false,"isRecursive":false},"column":{"values":["*"],"isExcludes":false,"isRecursive":false},"table":{"values":[],"isExcludes":false,"isRecursive":false}},"policyItems":[{"accesses":[{"type":"select","isAllowed":true},{"type":"update","isAllowed":true},{"type":"create","isAllowed":true},{"type":"drop","isAllowed":true},{"type":"alter","isAllowed":true},{"type":"index","isAllowed":true},{"type":"lock","isAllowed":true},{"type":"all","isAllowed":true}],"users":[],"groups":[],"conditions":[],"delegateAdmin":true,"isEnabled":true}],"denyPolicyItems":[],"allowExceptions":[],"denyExceptions":[],"id":2,"isEnabled":true}'

mapping_hdfs_template = '{"serviceName":"cm_hdfs","resourceElements":{"path":{"values":[],"isExcludes":false,"isRecursive":true}},"resourceSignature":"eccc3391a63dc9944f002deb12dbe283ec04024bcc5fec54576c6a8c515d2f78","id":1,"isEnabled":true}'
mapping_hive_template = '{"serviceName":"cm_hive","resourceElements":{"database":{"values":["*"],"isExcludes":false,"isRecursive":false},"table":{"values":[],"isExcludes":false,"isRecursive":false}},"ownerUser":"","resourceSignature":"d4bb51fdd469526877ffb71896ea5d2e70f520bc6fbbcc693e4b015ef0363f27","additionalInfo":{"tableType":"EXTERNAL_TABLE"},"id":2,"isEnabled":true}'

mapping_template = '{"op":"add","sourceServiceName":"cm_hdfs","targetServiceName":"cm_hive","currentNotificationId":23469904,"previousNotificationId":23469895,"serviceResources":{},"mappings":{},"isDelta":false}'
request_template = '{"name":"","request":{"resource":{"elements":{"path":""}},"accessType":"","user":"","userGroups":[],"requestData":"","remoteIPAddress":"255.255.255.255"},"result":{"isAudited":true,"isAllowed":true,"policyId":1}}'


# Execute the parse_args() method
args = my_parser.parse_args()

def create_hive_policy_json():
    # Get policy template
    with open(args.inputPath+"/cm_hive_template.json") as jsonFile:
        data = json.load(jsonFile)

    # Add policies
    currentCycle = 1
    while currentCycle <= args.policyItems:
        temp_policy = json.loads(policy_item_hive_template)
        temp_policy['name']                     = "hive-test-policy-{}".format(currentCycle)
        temp_policy['id']                       = currentCycle
        temp_policy['policyItems'][0]['users']  = ["user{}".format(currentCycle)]
        temp_policy['policyItems'][0]['groups'] = ["group{}".format(currentCycle)]
        temp_policy['policyItems'][0]['roles']  = ["role{}".format(currentCycle)]

        temp_policy['resources']['table']['values'] = ["hive-table-".format(currentCycle)]

        data['policies'].append(temp_policy)
        currentCycle += 1

    # Writing policy to console to sample.json
    with open(args.outputPath + "/cm_hive.json", "w") as outfile:
        json.dump(data, outfile)

def create_policy_map_json():
    # Get policy template
    data = json.loads(mapping_template)

    # Add policies
    currentCycle = 1
    while currentCycle <= args.policyItems:
        # serviceResources 
        temp_hdfs = json.loads(mapping_hdfs_template)
        temp_hive = json.loads(mapping_hive_template)

        ## id
        temp_hdfs['id'] = currentCycle
        temp_hive['id'] = args.policyItems + currentCycle

        temp_hdfs['resourceElements']['path']['values']  = ['/test/warehouse/hive-table-{}'.format(currentCycle)]
        temp_hive['resourceElements']['table']['values'] = ['hive-table-{}'.format(currentCycle)]

        data['serviceResources']["{}".format(temp_hdfs['id'])] = temp_hdfs
        data['serviceResources']["{}".format(temp_hive['id'])] = temp_hive
        
        # mappings
        data['mappings']["{}".format(temp_hdfs['id'])] = [temp_hive['id']]
        
        currentCycle += 1

    # Writing policy to console to sample.json
    with open(args.outputPath + "/resource_mapping.json", "w") as outfile:
        json.dump(data, outfile)


def get_test_request(data, result, access, path, user, policy_id):
    data['name']                   = "{} '{} {}' for u={}".format(result, access, path, user)
    data['request']['accessType']  = access
    data['request']['user']        = user
    data['request']['requestData'] = "{} {}".format(access, path)
    data['result']['isAllowed']    = True if "ALLOW".lower() == result.lower() else False
    data['result']['policyId']     = policy_id

    data['request']['resource']['elements']['path'] = path

    return data

def create_test_hdfs_request_json():
    # Get policy template
    data = json.loads('[]')

    # Add policies
    currentCycle = 1
    while currentCycle <= args.policyItems:
        temp_path  = '/test/warehouse/hive-table-{}'.format(currentCycle)
        user_allow = "user{}".format(currentCycle)
        user_deny  = "denyuser{}".format(currentCycle)

        # ALLOW read
        data.append(get_test_request(json.loads(request_template), 'ALLOW', 'read', temp_path, user_allow, 1))

        # ALLOW write
        data.append(get_test_request(json.loads(request_template), 'ALLOW', 'write', temp_path, user_allow, 1))

        # DENY  read
        data.append(get_test_request(json.loads(request_template), 'DENY', 'read', temp_path, user_deny, -1))

        # DENY  write
        data.append(get_test_request(json.loads(request_template), 'DENY', 'write', temp_path, user_allow, -1))        
        
        currentCycle += 1

    # Writing policy to console to sample.json
    with open(args.outputPath + "/test_requests_hdfs.json", "w") as outfile:
        json.dump(data, outfile)


# Create cm_hive.json file
create_hive_policy_json()

# Create mapping file
create_policy_map_json()

# Create cm_hive.json file
create_test_hdfs_request_json()
